@@ -29,8 +29,6 @@
 /*	#define TLS_INTERNAL
 /*	#include <tls.h>
 /*
-/*	char	*var_tls_cnf_file;
-/*	char	*var_tls_cnf_name;
 /*	char	*var_tls_high_clist;
 /*	char	*var_tls_medium_clist;
 /*	char	*var_tls_low_clist;
@@ -71,12 +69,8 @@
 /*
 /*	void	tls_param_init()
 /*
-/*	int     tls_library_init(void)
-/*
-/*	int	tls_proto_mask_lims(plist, floor, ceiling)
+/*	int	tls_protocol_mask(plist)
 /*	const char *plist;
-/*	int	*floor;
-/*	int	*ceiling;
 /*
 /*	int	tls_cipher_grade(name)
 /*	const char *name;
@@ -115,7 +109,7 @@
 /*	TLS_APPL_STATE *app_ctx;
 /*	int	log_mask;
 /*
-/*	const EVP_MD *tls_validate_digest(dgst)
+/*	int	tls_validate_digest(dgst)
 /*	const char *dgst;
 /* DESCRIPTION
 /*	This module implements public and internal routines that
@@ -159,19 +153,14 @@
 /*	tls_param_init() loads main.cf parameters used internally in
 /*	TLS library. Any errors are fatal.
 /*
-/*	tls_library_init() initializes the OpenSSL library, optionally
-/*	loading an OpenSSL configuration file.
-/*
 /*	tls_pre_jail_init() opens any tables that need to be opened before
 /*	entering a chroot jail. The "role" parameter must be TLS_ROLE_CLIENT
 /*	for clients and TLS_ROLE_SERVER for servers. Any errors are fatal.
 /*
-/*	tls_proto_mask_lims() returns a bitmask of excluded protocols, and
-/*	and the protocol version floor/ceiling, given a list (plist) of
-/*	protocols to include or (preceded by a '!') exclude, or constraints
-/*	of the form '>=name', '<=name', '>=hexvalue', '<=hexvalue'. If "plist"
-/*	contains invalid protocol names, TLS_PROTOCOL_INVALID is returned and
-/*	no warning is logged.
+/*	tls_protocol_mask() returns a bitmask of excluded protocols, given
+/*	a list (plist) of protocols to include or (preceded by a '!') exclude.
+/*	If "plist" contains invalid protocol names, TLS_PROTOCOL_INVALID is
+/*	returned and no warning is logged.
 /*
 /*	tls_cipher_grade() converts a case-insensitive cipher grade
 /*	name (high, medium, low, export, null) to the corresponding
@@ -213,8 +202,8 @@
 /*	tls_update_app_logmask() changes the log mask of the
 /*	application TLS context to the new setting.
 /*
-/*	tls_validate_digest() returns a static handle for the named
-/*	digest algorithm, or NULL on error.
+/*	tls_validate_digest() returns non-zero if the named digest
+/*	is usable and zero otherwise.
 /* LICENSE
 /* .ad
 /* .fi
@@ -250,6 +239,8 @@
 #include <ctype.h>
 #include <string.h>
 
+#ifdef USE_TLS
+
 /* Utility library. */
 
 #include <vstream.h>
@@ -281,8 +272,6 @@
  /*
   * Tunable parameters.
   */
-char   *var_tls_cnf_file;
-char   *var_tls_cnf_name;
 char   *var_tls_high_clist;
 char   *var_tls_medium_clist;
 char   *var_tls_low_clist;
@@ -303,12 +292,14 @@ char   *var_tls_tkt_cipher;
 char   *var_openssl_path;
 char   *var_tls_server_sni_maps;
 bool    var_tls_fast_shutdown;
-bool    var_tls_preempt_clist;
-
-#ifdef USE_TLS
 
 static MAPS *tls_server_sni_maps;
 
+#ifdef VAR_TLS_PREEMPT_CLIST
+bool    var_tls_preempt_clist;
+
+#endif
+
  /*
   * Index to attach TLScontext pointers to SSL objects, so that they can be
   * accessed by call-back routines.
@@ -328,19 +319,6 @@ static const NAME_CODE protocol_table[] = {
     0, TLS_PROTOCOL_INVALID,
 };
 
-/*
- * Protocol name => numeric version, for MinProtocol and MaxProtocol
- */
-static const NAME_CODE tls_version_table[] = {
-    "None", 0,
-    SSL_TXT_SSLV3, SSL3_VERSION,
-    SSL_TXT_TLSV1, TLS1_VERSION,
-    SSL_TXT_TLSV1_1, TLS1_1_VERSION,
-    SSL_TXT_TLSV1_2, TLS1_2_VERSION,
-    TLS_PROTOCOL_TXT_TLSV1_3, TLS1_3_VERSION,
-    0, -1,
-};
-
  /*
   * SSL_OP_MUMBLE bug work-around name <=> mask conversion.
   */
@@ -541,7 +519,6 @@ static const NAME_MASK tls_log_table[] = {
     "certmatch", TLS_LOG_CERTMATCH,
     "verbose", TLS_LOG_VERBOSE,		/* Postfix TLS library verbose */
     "cache", TLS_LOG_CACHE,
-    "dane", TLS_LOG_DANE,		/* DANE policy construction */
     "ssl-debug", TLS_LOG_DEBUG,		/* SSL library debug/verbose */
     "ssl-handshake-packet-dump", TLS_LOG_TLSPKTS,
     "ssl-session-packet-dump", TLS_LOG_TLSPKTS | TLS_LOG_ALLPKTS,
@@ -577,32 +554,9 @@ void    tls_update_app_logmask(TLS_APPL_STATE *app_ctx, int log_mask)
     app_ctx->log_mask = log_mask;
 }
 
-/* parse_version - parse TLS protocol version name or hex number */
+/* tls_protocol_mask - Bitmask of protocols to exclude */
 
-static int parse_tls_version(const char *tok, int *version)
-{
-    int     code = name_code(tls_version_table, NAME_CODE_FLAG_NONE, tok);
-    char   *_end;
-    unsigned long ulval;
-
-    if (code != -1) {
-	*version = code;
-	return (0);
-    }
-    errno = 0;
-    ulval = strtoul(tok, &_end, 16);
-    if (*_end != 0
-	|| (ulval == ULONG_MAX && errno == ERANGE)
-	|| ulval > INT_MAX)
-	return TLS_PROTOCOL_INVALID;
-
-    *version = (int) ulval;
-    return (0);
-}
-
-/* tls_proto_mask_lims - protocols to exclude and floor/ceiling */
-
-int     tls_proto_mask_lims(const char *plist, int *floor, int *ceiling)
+int     tls_protocol_mask(const char *plist)
 {
     char   *save;
     char   *tok;
@@ -616,15 +570,9 @@ int     tls_proto_mask_lims(const char *plist, int *floor, int *ceiling)
 	return (res); \
     } while (0)
 
-    *floor = *ceiling = 0;
-
     save = cp = mystrdup(plist);
     while ((tok = mystrtok(&cp, CHARS_COMMA_SP ":")) != 0) {
-	if (strncmp(tok, ">=", 2) == 0)
-	    code = parse_tls_version(tok + 2, floor);
-	else if (strncmp(tok, "<=", 2) == 0)
-	    code = parse_tls_version(tok + 2, ceiling);
-	else if (*tok == '!')
+	if (*tok == '!')
 	    exclude |= code =
 		name_code(protocol_table, NAME_CODE_FLAG_NONE, ++tok);
 	else
@@ -651,16 +599,14 @@ void    tls_param_init(void)
 {
     /* If this changes, update TLS_CLIENT_PARAMS in tls_proxy.h. */
     static const CONFIG_STR_TABLE str_table[] = {
-	VAR_TLS_CNF_FILE, DEF_TLS_CNF_FILE, &var_tls_cnf_file, 0, 0,
-	VAR_TLS_CNF_NAME, DEF_TLS_CNF_NAME, &var_tls_cnf_name, 0, 0,
 	VAR_TLS_HIGH_CLIST, DEF_TLS_HIGH_CLIST, &var_tls_high_clist, 1, 0,
 	VAR_TLS_MEDIUM_CLIST, DEF_TLS_MEDIUM_CLIST, &var_tls_medium_clist, 1, 0,
 	VAR_TLS_LOW_CLIST, DEF_TLS_LOW_CLIST, &var_tls_low_clist, 1, 0,
 	VAR_TLS_EXPORT_CLIST, DEF_TLS_EXPORT_CLIST, &var_tls_export_clist, 1, 0,
 	VAR_TLS_NULL_CLIST, DEF_TLS_NULL_CLIST, &var_tls_null_clist, 1, 0,
-	VAR_TLS_EECDH_AUTO, DEF_TLS_EECDH_AUTO, &var_tls_eecdh_auto, 0, 0,
-	VAR_TLS_EECDH_STRONG, DEF_TLS_EECDH_STRONG, &var_tls_eecdh_strong, 0, 0,
-	VAR_TLS_EECDH_ULTRA, DEF_TLS_EECDH_ULTRA, &var_tls_eecdh_ultra, 0, 0,
+	VAR_TLS_EECDH_AUTO, DEF_TLS_EECDH_AUTO, &var_tls_eecdh_auto, 1, 0,
+	VAR_TLS_EECDH_STRONG, DEF_TLS_EECDH_STRONG, &var_tls_eecdh_strong, 1, 0,
+	VAR_TLS_EECDH_ULTRA, DEF_TLS_EECDH_ULTRA, &var_tls_eecdh_ultra, 1, 0,
 	VAR_TLS_BUG_TWEAKS, DEF_TLS_BUG_TWEAKS, &var_tls_bug_tweaks, 0, 0,
 	VAR_TLS_SSL_OPTIONS, DEF_TLS_SSL_OPTIONS, &var_tls_ssl_options, 0, 0,
 	VAR_TLS_DANE_DIGESTS, DEF_TLS_DANE_DIGESTS, &var_tls_dane_digests, 1, 0,
@@ -696,118 +642,6 @@ void    tls_param_init(void)
     get_mail_conf_bool_table(bool_table);
 }
 
-/* tls_library_init - perform OpenSSL library initialization */
-
-int     tls_library_init(void)
-{
-    OPENSSL_INIT_SETTINGS *init_settings;
-    char   *conf_name = *var_tls_cnf_name ? var_tls_cnf_name : 0;
-    char   *conf_file = 0;
-    unsigned long init_opts = 0;
-
-#define TLS_LIB_INIT_TODO	(-1)
-#define TLS_LIB_INIT_ERR	(0)
-#define TLS_LIB_INIT_OK		(1)
-
-    static int init_res = TLS_LIB_INIT_TODO;
-
-    if (init_res != TLS_LIB_INIT_TODO)
-	return (init_res);
-
-    /*
-     * Backwards compatibility: skip this function unless the Postfix
-     * configuration actually has non-default tls_config_xxx settings.
-     */
-    if (strcmp(var_tls_cnf_file, DEF_TLS_CNF_FILE) == 0
-	&& strcmp(var_tls_cnf_name, DEF_TLS_CNF_NAME) == 0) {
-	if (msg_verbose)
-	    msg_info("tls_library_init: using backwards-compatible defaults");
-	return (init_res = TLS_LIB_INIT_OK);
-    }
-    if ((init_settings = OPENSSL_INIT_new()) == 0) {
-	msg_warn("error allocating OpenSSL init settings, "
-		 "disabling TLS support");
-	return (init_res = TLS_LIB_INIT_ERR);
-    }
-#define TLS_LIB_INIT_RETURN(x) \
-    do { OPENSSL_INIT_free(init_settings); return (init_res = (x)); } while(0)
-
-#if OPENSSL_VERSION_NUMBER < 0x1010102fL
-
-    /*
-     * OpenSSL 1.1.0 through 1.1.1a, no support for custom configuration
-     * files, disabling loading of the file, or getting strict error
-     * handling.  Thus, the only supported configuration file is "default".
-     */
-    if (strcmp(var_tls_cnf_file, "default") != 0) {
-	msg_warn("non-default %s = %s requires OpenSSL 1.1.1b or later, "
-	       "disabling TLS support", VAR_TLS_CNF_FILE, var_tls_cnf_file);
-	TLS_LIB_INIT_RETURN(TLS_LIB_INIT_ERR);
-    }
-#else
-    {
-	unsigned long file_flags = 0;
-
-	/*-
-	 * OpenSSL 1.1.1b or later:
-	 * We can now use a non-default configuration file, or
-	 * use none at all.  We can also request strict error
-	 * reporting.
-	 */
-	if (strcmp(var_tls_cnf_file, "none") == 0) {
-	    init_opts |= OPENSSL_INIT_NO_LOAD_CONFIG;
-	} else if (strcmp(var_tls_cnf_file, "default") == 0) {
-
-	    /*
-	     * The default global config file is optional.  With "default"
-	     * initialisation we don't insist on a match for the requested
-	     * application name, allowing fallback to the default application
-	     * name, even when a non-default application name is specified.
-	     * Errors in loading the default configuration are ignored.
-	     */
-	    conf_file = 0;
-	    file_flags |= CONF_MFLAGS_IGNORE_MISSING_FILE;
-	    file_flags |= CONF_MFLAGS_DEFAULT_SECTION;
-	    file_flags |= CONF_MFLAGS_IGNORE_RETURN_CODES | CONF_MFLAGS_SILENT;
-	} else if (*var_tls_cnf_file == '/') {
-
-	    /*
-	     * A custom config file must be present, error reporting is
-	     * strict and the configuration section for the requested
-	     * application name does not fall back to "openssl_conf" when
-	     * missing.
-	     */
-	    conf_file = var_tls_cnf_file;
-	} else {
-	    msg_warn("non-default %s = %s is not an absolute pathname, "
-	       "disabling TLS support", VAR_TLS_CNF_FILE, var_tls_cnf_file);
-	    TLS_LIB_INIT_RETURN(TLS_LIB_INIT_ERR);
-	}
-
-	OPENSSL_INIT_set_config_file_flags(init_settings, file_flags);
-    }
-#endif
-
-    if (conf_file)
-	OPENSSL_INIT_set_config_filename(init_settings, conf_file);
-    if (conf_name)
-	OPENSSL_INIT_set_config_appname(init_settings, conf_name);
-
-    if (OPENSSL_init_ssl(init_opts, init_settings) <= 0) {
-	if ((init_opts & OPENSSL_INIT_NO_LOAD_CONFIG) == 0)
-	    msg_warn("error loading the '%s' settings from the %s OpenSSL "
-		     "configuration file, disabling TLS support",
-		     conf_name ? conf_name : "global",
-		     conf_file ? conf_file : "default");
-	else
-	    msg_warn("error initializing the OpenSSL library, "
-		     "disabling TLS support");
-	tls_print_errors();
-	TLS_LIB_INIT_RETURN(TLS_LIB_INIT_ERR);
-    }
-    TLS_LIB_INIT_RETURN(TLS_LIB_INIT_OK);
-}
-
 /* tls_pre_jail_init - Load TLS related pre-jail tables */
 
 void    tls_pre_jail_init(TLS_ROLE role)
@@ -988,6 +822,7 @@ const char *tls_set_ciphers(TLS_SESS_STATE *TLScontext, const char *grade,
 
 void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 {
+#if OPENSSL_VERSION_NUMBER >= 0x1010100fUL && defined(TLS1_3_VERSION)
     const char *kex_name = 0;
     const char *kex_curve = 0;
     const char *locl_sig_name = 0;
@@ -999,14 +834,11 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
     int     nid;
     SSL    *ssl = TLScontext->con;
     int     srvr = SSL_is_server(ssl);
-    EVP_PKEY *dh_pkey = 0;
-    X509   *local_cert;
-    EVP_PKEY *local_pkey = 0;
-    X509   *peer_cert;
-    EVP_PKEY *peer_pkey = 0;
+    X509   *cert;
+    EVP_PKEY *pkey = 0;
 
 #ifndef OPENSSL_NO_EC
-    const EC_KEY *eckey;
+    EC_KEY *eckey;
 
 #endif
 
@@ -1015,27 +847,21 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
     if (SSL_version(ssl) < TLS1_3_VERSION)
 	return;
 
-    if (tls_get_peer_dh_pubkey(ssl, &dh_pkey)) {
-	switch (nid = EVP_PKEY_id(dh_pkey)) {
+    if (tls_get_peer_dh_pubkey(ssl, &pkey)) {
+	switch (nid = EVP_PKEY_id(pkey)) {
 	default:
 	    kex_name = OBJ_nid2sn(EVP_PKEY_type(nid));
 	    break;
 
-#if defined(EVP_PKEY_KEYMGMT)
-	case EVP_PKEY_KEYMGMT:
-	    kex_name = EVP_PKEY_get0_type_name(dh_pkey);
-	    break;
-#endif
-
 	case EVP_PKEY_DH:
 	    kex_name = "DHE";
-	    TLScontext->kex_bits = EVP_PKEY_bits(dh_pkey);
+	    TLScontext->kex_bits = EVP_PKEY_bits(pkey);
 	    break;
 
 #ifndef OPENSSL_NO_EC
 	case EVP_PKEY_EC:
 	    kex_name = "ECDHE";
-	    eckey = EVP_PKEY_get0_EC_KEY(dh_pkey);
+	    eckey = EVP_PKEY_get0_EC_KEY(pkey);
 	    nid = EC_GROUP_get_curve_name(EC_KEY_get0_group(eckey));
 	    kex_curve = EC_curve_nid2nist(nid);
 	    if (!kex_curve)
@@ -1043,18 +869,8 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 	    break;
 #endif
 	}
+	EVP_PKEY_free(pkey);
     }
-    if (kex_name) {
-	TLScontext->kex_name = mystrdup(kex_name);
-        if (kex_curve)
-            TLScontext->kex_curve = mystrdup(kex_curve);
-    }
-    /* Not a problem if NULL */
-    EVP_PKEY_free(dh_pkey);
-
-    /* Resumption makes no use of signature keys or digests */
-    if (TLScontext->session_reused)
-	return;
 
     /*
      * On the client end, the certificate may be preset, but not used, so we
@@ -1062,42 +878,35 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
      * data on clients requires at least 1.1.1a.
      */
     if (srvr || SSL_get_signature_nid(ssl, &nid))
-	local_cert = SSL_get_certificate(ssl);
+	cert = SSL_get_certificate(ssl);
     else
-	local_cert = 0;
+	cert = 0;
 
     /* Signature algorithms for the local end of the connection */
-    if (local_cert) {
-	local_pkey = X509_get0_pubkey(local_cert);
+    if (cert) {
+	pkey = X509_get0_pubkey(cert);
 
 	/*
 	 * Override the built-in name for the "ECDSA" algorithms OID, with
 	 * the more familiar name.  For "RSA" keys report "RSA-PSS", which
 	 * must be used with TLS 1.3.
 	 */
-	if ((nid = EVP_PKEY_id(local_pkey)) != NID_undef) {
+	if ((nid = EVP_PKEY_type(EVP_PKEY_id(pkey))) != NID_undef) {
 	    switch (nid) {
 	    default:
-		if ((nid = EVP_PKEY_type(nid)) != NID_undef)
-		    locl_sig_name = OBJ_nid2sn(nid);
+		locl_sig_name = OBJ_nid2sn(nid);
 		break;
 
-#if defined(EVP_PKEY_KEYMGMT)
-	    case EVP_PKEY_KEYMGMT:
-		locl_sig_name = EVP_PKEY_get0_type_name(local_pkey);
-		break;
-#endif
-
 	    case EVP_PKEY_RSA:
 		/* For RSA, TLS 1.3 mandates PSS signatures */
 		locl_sig_name = "RSA-PSS";
-		SIG_PROP(TLScontext, srvr, bits) = EVP_PKEY_bits(local_pkey);
+		SIG_PROP(TLScontext, srvr, bits) = EVP_PKEY_bits(pkey);
 		break;
 
 #ifndef OPENSSL_NO_EC
 	    case EVP_PKEY_EC:
 		locl_sig_name = "ECDSA";
-		eckey = EVP_PKEY_get0_EC_KEY(local_pkey);
+		eckey = EVP_PKEY_get0_EC_KEY(pkey);
 		nid = EC_GROUP_get_curve_name(EC_KEY_get0_group(eckey));
 		locl_sig_curve = EC_curve_nid2nist(nid);
 		if (!locl_sig_curve)
@@ -1105,7 +914,6 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 		break;
 #endif
 	    }
-	    /* No X509_free(local_cert) */
 	}
 
 	/*
@@ -1114,47 +922,32 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 	 */
 	if (SSL_get_signature_nid(ssl, &nid) && nid != NID_undef)
 	    locl_sig_dgst = OBJ_nid2sn(nid);
-
-	if (locl_sig_name) {
-	    SIG_PROP(TLScontext, srvr, name) = mystrdup(locl_sig_name);
-	    if (locl_sig_curve)
-		SIG_PROP(TLScontext, srvr, curve) = mystrdup(locl_sig_curve);
-	    if (locl_sig_dgst)
-		SIG_PROP(TLScontext, srvr, dgst) = mystrdup(locl_sig_dgst);
-	}
     }
     /* Signature algorithms for the peer end of the connection */
-    if ((peer_cert = SSL_get_peer_certificate(ssl)) != 0) {
-	peer_pkey = X509_get0_pubkey(peer_cert);
+    if ((cert = SSL_get_peer_certificate(ssl)) != 0) {
+	pkey = X509_get0_pubkey(cert);
 
 	/*
 	 * Override the built-in name for the "ECDSA" algorithms OID, with
 	 * the more familiar name.  For "RSA" keys report "RSA-PSS", which
 	 * must be used with TLS 1.3.
 	 */
-	if ((nid = EVP_PKEY_id(peer_pkey)) != NID_undef) {
+	if ((nid = EVP_PKEY_type(EVP_PKEY_id(pkey))) != NID_undef) {
 	    switch (nid) {
 	    default:
-		if ((nid = EVP_PKEY_type(nid)) != NID_undef)
-		    peer_sig_name = OBJ_nid2sn(nid);
+		peer_sig_name = OBJ_nid2sn(nid);
 		break;
 
-#if defined(EVP_PKEY_KEYMGMT)
-	    case EVP_PKEY_KEYMGMT:
-		peer_sig_name = EVP_PKEY_get0_type_name(peer_pkey);
-		break;
-#endif
-
 	    case EVP_PKEY_RSA:
 		/* For RSA, TLS 1.3 mandates PSS signatures */
 		peer_sig_name = "RSA-PSS";
-		SIG_PROP(TLScontext, !srvr, bits) = EVP_PKEY_bits(peer_pkey);
+		SIG_PROP(TLScontext, !srvr, bits) = EVP_PKEY_bits(pkey);
 		break;
 
 #ifndef OPENSSL_NO_EC
 	    case EVP_PKEY_EC:
 		peer_sig_name = "ECDSA";
-		eckey = EVP_PKEY_get0_EC_KEY(peer_pkey);
+		eckey = EVP_PKEY_get0_EC_KEY(pkey);
 		nid = EC_GROUP_get_curve_name(EC_KEY_get0_group(eckey));
 		peer_sig_curve = EC_curve_nid2nist(nid);
 		if (!peer_sig_curve)
@@ -1170,16 +963,27 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 	 */
 	if (SSL_get_peer_signature_nid(ssl, &nid) && nid != NID_undef)
 	    peer_sig_dgst = OBJ_nid2sn(nid);
-
-	if (peer_sig_name) {
-	    SIG_PROP(TLScontext, !srvr, name) = mystrdup(peer_sig_name);
-	    if (peer_sig_curve)
-		SIG_PROP(TLScontext, !srvr, curve) = mystrdup(peer_sig_curve);
-	    if (peer_sig_dgst)
-		SIG_PROP(TLScontext, !srvr, dgst) = mystrdup(peer_sig_dgst);
-	}
-	X509_free(peer_cert);
     }
+    if (kex_name) {
+	TLScontext->kex_name = mystrdup(kex_name);
+	if (kex_curve)
+	    TLScontext->kex_curve = mystrdup(kex_curve);
+    }
+    if (locl_sig_name) {
+	SIG_PROP(TLScontext, srvr, name) = mystrdup(locl_sig_name);
+	if (locl_sig_curve)
+	    SIG_PROP(TLScontext, srvr, curve) = mystrdup(locl_sig_curve);
+	if (locl_sig_dgst)
+	    SIG_PROP(TLScontext, srvr, dgst) = mystrdup(locl_sig_dgst);
+    }
+    if (peer_sig_name) {
+	SIG_PROP(TLScontext, !srvr, name) = mystrdup(peer_sig_name);
+	if (peer_sig_curve)
+	    SIG_PROP(TLScontext, !srvr, curve) = mystrdup(peer_sig_curve);
+	if (peer_sig_dgst)
+	    SIG_PROP(TLScontext, !srvr, dgst) = mystrdup(peer_sig_dgst);
+    }
+#endif						/* OPENSSL_VERSION_NUMBER ... */
 }
 
 /* tls_log_summary - TLS loglevel 1 one-liner, embellished with TLS 1.3 details */
@@ -1317,8 +1121,11 @@ TLS_SESS_STATE *tls_alloc_sess_context(int log_mask, const char *namaddr)
     TLScontext->mdalg = 0;			/* Alias for props->mdalg */
     TLScontext->dane = 0;			/* Alias for props->dane */
     TLScontext->errordepth = -1;
+    TLScontext->tadepth = -1;
     TLScontext->errorcode = X509_V_OK;
     TLScontext->errorcert = 0;
+    TLScontext->untrusted = 0;
+    TLScontext->trusted = 0;
 
     return (TLScontext);
 }
@@ -1351,24 +1158,12 @@ void    tls_free_context(TLS_SESS_STATE *TLScontext)
 	myfree(TLScontext->peer_cert_fprint);
     if (TLScontext->peer_pkey_fprint)
 	myfree(TLScontext->peer_pkey_fprint);
-    if (TLScontext->kex_name)
-	myfree((void *) TLScontext->kex_name);
-    if (TLScontext->kex_curve)
-	myfree((void *) TLScontext->kex_curve);
-    if (TLScontext->clnt_sig_name)
-	myfree((void *) TLScontext->clnt_sig_name);
-    if (TLScontext->clnt_sig_curve)
-	myfree((void *) TLScontext->clnt_sig_curve);
-    if (TLScontext->clnt_sig_dgst)
-	myfree((void *) TLScontext->clnt_sig_dgst);
-    if (TLScontext->srvr_sig_name)
-	myfree((void *) TLScontext->srvr_sig_name);
-    if (TLScontext->srvr_sig_curve)
-	myfree((void *) TLScontext->srvr_sig_curve);
-    if (TLScontext->srvr_sig_dgst)
-	myfree((void *) TLScontext->srvr_sig_dgst);
     if (TLScontext->errorcert)
 	X509_free(TLScontext->errorcert);
+    if (TLScontext->untrusted)
+	sk_X509_pop_free(TLScontext->untrusted, X509_free);
+    if (TLScontext->trusted)
+	sk_X509_pop_free(TLScontext->trusted, X509_free);
 
     myfree((void *) TLScontext);
 }
@@ -1387,17 +1182,57 @@ static void tls_version_split(unsigned long version, TLS_VINFO *info)
      * 
      * The status nibble has one of the values 0 for development, 1 to e for
      * betas 1 to 14, and f for release. Parsed OpenSSL version number. for
-     * example: 0x1010103f == 1.1.1c.
+     * example
+     * 
+     * 0x000906000 == 0.9.6 dev 0x000906023 == 0.9.6b beta 3 0x00090605f ==
+     * 0.9.6e release
+     * 
+     * Versions prior to 0.9.3 have identifiers < 0x0930.  Versions between
+     * 0.9.3 and 0.9.5 had a version identifier with this interpretation:
+     * 
+     * MMNNFFRBB major minor fix final beta/patch
+     * 
+     * for example
+     * 
+     * 0x000904100 == 0.9.4 release 0x000905000 == 0.9.5 dev
+     * 
+     * Version 0.9.5a had an interim interpretation that is like the current
+     * one, except the patch level got the highest bit set, to keep continu-
+     * ity.  The number was therefore 0x0090581f.
      */
-    info->status = version & 0xf;
-    version >>= 4;
-    info->patch = version & 0xff;
-    version >>= 8;
-    info->micro = version & 0xff;
-    version >>= 8;
-    info->minor = version & 0xff;
-    version >>= 8;
-    info->major = version & 0xff;
+
+    if (version < 0x0930) {
+	info->status = 0;
+	info->patch = version & 0x0f;
+	version >>= 4;
+	info->micro = version & 0x0f;
+	version >>= 4;
+	info->minor = version & 0x0f;
+	version >>= 4;
+	info->major = version & 0x0f;
+    } else if (version < 0x00905800L) {
+	info->patch = version & 0xff;
+	version >>= 8;
+	info->status = version & 0xf;
+	version >>= 4;
+	info->micro = version & 0xff;
+	version >>= 8;
+	info->minor = version & 0xff;
+	version >>= 8;
+	info->major = version & 0xff;
+    } else {
+	info->status = version & 0xf;
+	version >>= 4;
+	info->patch = version & 0xff;
+	version >>= 8;
+	info->micro = version & 0xff;
+	version >>= 8;
+	info->minor = version & 0xff;
+	version >>= 8;
+	info->major = version & 0xff;
+	if (version < 0x00906000L)
+	    info->patch &= ~0x80;
+    }
 }
 
 /* tls_check_version - Detect mismatch between headers and library. */
@@ -1659,17 +1494,47 @@ long    tls_bio_dump_cb(BIO *bio, int cmd, const char *argp, int argi,
     return (ret);
 }
 
-const EVP_MD *tls_validate_digest(const char *dgst)
+int     tls_validate_digest(const char *dgst)
 {
     const EVP_MD *md_alg;
+    unsigned int md_len;
+
+    /*
+     * Register SHA-2 digests, if implemented and not already registered.
+     * Improves interoperability with clients and servers that prematurely
+     * deploy SHA-2 certificates.  Also facilitates DANE and TA support.
+     */
+#if defined(LN_sha256) && defined(NID_sha256) && !defined(OPENSSL_NO_SHA256)
+    if (!EVP_get_digestbyname(LN_sha224))
+	EVP_add_digest(EVP_sha224());
+    if (!EVP_get_digestbyname(LN_sha256))
+	EVP_add_digest(EVP_sha256());
+#endif
+#if defined(LN_sha512) && defined(NID_sha512) && !defined(OPENSSL_NO_SHA512)
+    if (!EVP_get_digestbyname(LN_sha384))
+	EVP_add_digest(EVP_sha384());
+    if (!EVP_get_digestbyname(LN_sha512))
+	EVP_add_digest(EVP_sha512());
+#endif
 
     /*
      * If the administrator specifies an unsupported digest algorithm, fail
      * now, rather than in the middle of a TLS handshake.
      */
-    if ((md_alg = tls_digest_byname(dgst, NULL)) == 0)
+    if ((md_alg = EVP_get_digestbyname(dgst)) == 0) {
 	msg_warn("Digest algorithm \"%s\" not found", dgst);
-    return md_alg;
+	return (0);
+    }
+
+    /*
+     * Sanity check: Newer shared libraries may use larger digests.
+     */
+    if ((md_len = EVP_MD_size(md_alg)) > EVP_MAX_MD_SIZE) {
+	msg_warn("Digest algorithm \"%s\" output size %u too large",
+		 dgst, md_len);
+	return (0);
+    }
+    return (1);
 }
 
 #else
