@@ -781,6 +781,7 @@ static void tlsp_strategy(TLSP_STATE *state)
 	 */
 	if (state->flags & TLSP_FLAG_DO_HANDSHAKE) {
 	    state->timeout = state->handshake_timeout;
+	    ERR_clear_error();
 	    if (state->is_server_role)
 		ssl_stat = SSL_accept(tls_context->con);
 	    else
@@ -809,6 +810,7 @@ static void tlsp_strategy(TLSP_STATE *state)
 	if (NBBIO_ERROR_FLAGS(plaintext_buf)) {
 	    if (NBBIO_ACTIVE_FLAGS(plaintext_buf))
 		nbbio_disable_readwrite(state->plaintext_buf);
+	    ERR_clear_error();
 	    if (!SSL_in_init(tls_context->con)
 		&& (ssl_stat = SSL_shutdown(tls_context->con)) < 0) {
 		handshake_err = SSL_get_error(tls_context->con, ssl_stat);
@@ -838,6 +840,7 @@ static void tlsp_strategy(TLSP_STATE *state)
 	 */
 	ssl_write_err = SSL_ERROR_NONE;
 	while (NBBIO_READ_PEND(plaintext_buf) > 0) {
+	    ERR_clear_error();
 	    ssl_stat = SSL_write(tls_context->con, NBBIO_READ_BUF(plaintext_buf),
 				 NBBIO_READ_PEND(plaintext_buf));
 	    ssl_write_err = SSL_get_error(tls_context->con, ssl_stat);
@@ -870,6 +873,7 @@ static void tlsp_strategy(TLSP_STATE *state)
 	 */
 	ssl_read_err = SSL_ERROR_NONE;
 	while (NBBIO_WRITE_PEND(state->plaintext_buf) < NBBIO_BUFSIZE(plaintext_buf)) {
+	    ERR_clear_error();
 	    ssl_stat = SSL_read(tls_context->con,
 				NBBIO_WRITE_BUF(plaintext_buf)
 				+ NBBIO_WRITE_PEND(state->plaintext_buf),
@@ -993,12 +997,12 @@ static int tlsp_client_start_pre_handshake(TLSP_STATE *state)
     state->client_start_props->ctx = state->appl_state;
     state->client_start_props->fd = state->ciphertext_fd;
     /* These predicates and warning belong inside tls_client_start(). */
-    if (!TLS_DANE_BASED(state->client_start_props->tls_level)
-	|| tls_dane_avail())
-	state->tls_context = tls_client_start(state->client_start_props);
-    else
+    if (!tls_dane_avail()			/* mandatory side effects!! */
+	&&TLS_DANE_BASED(state->client_start_props->tls_level))
 	msg_warn("%s: DANE requested, but not available",
 		 state->client_start_props->namaddr);
+    else
+	state->tls_context = tls_client_start(state->client_start_props);
     if (state->tls_context != 0)
 	return (TLSP_STAT_OK);
 
